/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* curses misc stuff
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"

PUBLIC WINDOW *win_gram_b,
              *win_gram,
              *win_log_b,
              *win_log;
PUBLIC int    curses_f,
              win_ysize_gram,
              win_ysize_gram_len,
              win_ysize_log;

/***************************************************************************
*
***************************************************************************/
PUBLIC void curses_error(const char *func, char *msg)
{
  log_e(func, msg);
  exit(-1);
}

PUBLIC void win_log_m(char type, const char *func, char *msg)
{
  static int i;
  char       str[STR_LEN + 1];

  log_m_file(type, func, msg);

  if(msg[0] == 'E')
    wattrset(win_log, COLOR_PAIR(COLOUR_WHITE));
  else if(msg[0] == 'W')
    wattrset(win_log, COLOR_PAIR(COLOUR_CYAN));
  else
  {
    if(i)
      wattrset(win_log, COLOR_PAIR(COLOUR_RED));
    else
      wattrset(win_log, COLOR_PAIR(COLOUR_MAGENTA));
    i ^= 1;
  }

  if(func == NULL)
    sprintf(str, "\n%c%s", type, msg);
  else
    sprintf(str, "\n%c %s()%s", type, func, msg);
  waddstr(win_log, str);
  wrefresh(win_log);
}


/***************************************************************************
*
***************************************************************************/
PUBLIC void create_window(
  char *title,
  WINDOW **wb, WINDOW **w, int nlines, int ncols, int begin_y, int begin_x)
{
  *wb = newwin(nlines, ncols, begin_y, begin_x);
  if(*wb == NULL)
    curses_error(__func__, "::newwin(wb)");
  wattrset(*wb, COLOR_PAIR(COLOUR_NORMAL));
  box(*wb, ACS_VLINE, ACS_HLINE);
  mvwaddstr(*wb, 0, 2, title);

  *w = derwin(*wb, nlines - 2, ncols - 2, 1, 1);
  if(*w == NULL)
    curses_error(__func__, "::newwin(w)");
  scrollok(*w, 1);

  refresh();
  wrefresh(*wb);
  wrefresh(*w);
}

PUBLIC void init_curses(void)
{
  initscr();
  curses_f = 1;

  if(LINES < 23)
  {
    log_m('E', NULL, " screen too small. unable to run curses");
    exit(-1);
  }
  start_color();

  init_pair(COLOUR_NONE,    COLOR_BLACK,   COLOR_BLACK);
  init_pair(COLOUR_TOPBAR,  COLOR_BLUE,    COLOR_WHITE);
  init_pair(COLOUR_TOPBAR2, COLOR_BLUE,    COLOR_YELLOW);
  init_pair(COLOUR_TOPBARO, COLOR_GREEN,   COLOR_WHITE);
  init_pair(COLOUR_TOPBAR2O,COLOR_GREEN,   COLOR_YELLOW);
  init_pair(COLOUR_NORMAL,  COLOR_WHITE,   COLOR_BLACK);
  init_pair(COLOUR_HELP,    COLOR_MAGENTA, COLOR_BLACK);
  init_pair(COLOUR_ALERT,   COLOR_YELLOW,  COLOR_RED);
  init_pair(COLOUR_WHITE,   COLOR_WHITE,   COLOR_BLACK);
  init_pair(COLOUR_RED,     COLOR_RED,     COLOR_BLACK);
  init_pair(COLOUR_BLUE,    COLOR_BLUE,    COLOR_BLACK);
  init_pair(COLOUR_YELLOW,  COLOR_YELLOW,  COLOR_BLACK);
  init_pair(COLOUR_CYAN,    COLOR_CYAN,    COLOR_BLACK);
  init_pair(COLOUR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
  init_pair(COLOUR_GREEN,   COLOR_GREEN,   COLOR_BLACK);

  keypad(stdscr, TRUE);
  meta(stdscr, TRUE);
  nonl();
  cbreak();
  noecho();

  log_m = win_log_m;
  win_ysize_log = LINES - 4;
  win_ysize_gram = win_ysize_log - 2;
  win_ysize_gram_len = win_ysize_gram - 2;
  create_window("Grams", &win_gram_b, &win_gram, win_ysize_gram, 57,  1, 0);
  create_window("Log", &win_log_b, &win_log,     win_ysize_log, 23,  1, 57);
}

PUBLIC void curses_deinit(void)
{
  if(win_gram_b != NULL)
    delwin(win_gram);

  if(win_log_b != NULL)
    delwin(win_log_b);

  endwin();
}
